----------------------------------------------------------------------------------
-- Engineer: Catherine Lagunzad
-- Create Date: 11/27/2017 09:48:57 PM
-- Description: Counts down from 20 or 5 seconds depending on user input
-- once the output reaches 0, it will output '1' to turn the speaker on.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity down_counter_FSM is
    Port ( SEC_CLK : in STD_LOGIC;
           reset : in STD_LOGIC;
           count_down_20_sec : in STD_LOGIC;
           count_down_5_sec : in STD_LOGIC;
           counter_out : out STD_LOGIC_VECTOR(7 downto 0);
           valid_count : out STD_LOGIC;
           buzzer_timeout : out STD_LOGIC);
end down_counter_FSM;

architecture Behavioral of down_counter_FSM is
    type state_type is (START, ST20, ST19, ST18, ST17, ST16, ST15, ST14, ST13, ST12, 
    ST11, ST10, ST9, ST8, ST7, ST6, ST5, ST4, ST3, ST2, ST1, ST0);
    signal PS, NS : state_type;
    signal temp_count : STD_LOGIC_VECTOR(7 downto 0);

begin
    
    sync_proc : process (reset, SEC_CLK, NS)
    begin
        if (reset = '1') then
            PS <= START;
        elsif (rising_edge(SEC_CLK)) then
            PS <= NS;
        end if;
    end process;
    
    comb_proc : process (PS, count_down_20_sec, count_down_5_sec)
    begin
        case PS is
            when START =>
                buzzer_timeout <= '0';
                valid_count <= '0';
                if (count_down_20_sec = '0' AND count_down_5_sec = '0') then
                    NS <= START;
                elsif (count_down_20_sec = '0' OR count_down_5_sec = '1') then
                    valid_count <= '1';
                    NS <= ST5;
                elsif (count_down_20_sec = '1' OR count_down_5_sec = '0') then
                    valid_count <= '1';
                    NS <= ST20;
                end if;
                
            when ST20 =>
                    NS <= ST19;
            when ST19 =>
                    NS <= ST18;
            when ST18 =>
                    NS <= ST17;
            when ST17 =>
                    NS <= ST16;
            when ST16 =>
                    NS <= ST15;
            when ST15 =>
                    NS <= ST14;
            when ST14 =>
                    NS <= ST13;
            when ST13 =>
                    NS <= ST12;
            when ST12 =>
                    NS <= ST11;
            when ST11 =>
                    NS <= ST10;
            when ST10 =>
                    NS <= ST9;
            when ST9 =>
                    NS <= ST8;
            when ST8 =>
                    NS <= ST7;
            when ST7 =>
                    NS <= ST6;
            when ST6 =>
                    NS <= ST5;
            when ST5 =>
                    NS <= ST4;
            when ST4 =>
                    NS <= ST3;
            when ST3 =>
                    NS <= ST2;
            when ST2 =>
                    NS <= ST1;
            when ST1 =>
                    NS <= ST0;
            when ST0 =>
                buzzer_timeout <= '1';
        end case;
    end process;
    
    with PS select
        counter_out <= "00010100" when ST20,
                       "00010011" when ST19,
                       "00010010" when ST18,
                       "00010001" when ST17,
                       "00010000" when ST16,
                       "00001111" when ST15,
                       "00001110" when ST14,
                       "00001101" when ST13,
                       "00001100" when ST12,
                       "00001011" when ST11,
                       "00001010" when ST10,
                       "00001001" when ST9,
                       "00001000" when ST8,
                       "00000111" when ST7,
                       "00000110" when ST6,
                       "00000101" when ST5,
                       "00000100" when ST4,
                       "00000011" when ST3,
                       "00000010" when ST2,
                       "00000001" when ST1,
                       "00000000" when ST0,
                       "00000000" when others;

end Behavioral;